----------------------------------------------------------------------------------
-- Krmilnik za tipkovnico po standardu PS/2
----------------------------------------------------------------------------------
-- Glavni modul, ki skrbi tudi za sinhronizacijo vhodov KCLK in KDATA in vpis 
-- prebranih podatkov v register SIPO (serial-in-parallel-out).
--
-- verzija: 2021-12-08 
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity PS2Controller is
    port (
        clk   : in  std_logic;
        rst   : in  std_logic;
        kclk  : in  std_logic;
        kdata : in  std_logic;
        data  : out std_logic_vector (8 downto 0); -- [PAR | B7 | ... | B1 | B0]
        eot   : out std_logic -- end-of-transmission: signalizira konec prenosa ene kode
    );
end entity;

architecture Behavioral of PS2Controller is

    -- KOMPONENTE
    component pulseGen is
        port (
            clk    : in  std_logic;
            rst    : in  std_logic;
            kclk_s : in  std_logic;
            pulse  : out std_logic);
    end component;

    component controlUnit is
        port (
            clk     : in  std_logic;
            rst     : in  std_logic;
            kdata_s : in  std_logic;
            pulse   : in  std_logic;
            we      : out std_logic;
            eot     : out std_logic
        );
    end component;

    -- NOTRANJI SIGNALI
    signal pulse                            : std_logic;
    signal kdata_1, kdata_s, kclk_1, kclk_s : std_logic;
    signal we                               : std_logic;
    signal SIPO                             : std_logic_vector(8 downto 0);
begin

    -- Prirejanje vhoda/izhoda
    data <= SIPO;

    -- Instanciranje 
    pulseGen_inst : pulseGen
        port map(
            clk    => clk,
            rst    => rst,
            kclk_s => kclk_s,
            pulse  => pulse
        );

    controlUnit_inst : controlUnit
        port map(
            clk     => clk,
            pulse   => pulse,
            rst     => rst,
            kdata_s => kdata_s,
            we      => we,
            eot     => eot
        );

    -- Sinhronizacija vhodnih signalov kclk in kdata: 
    -- peljemo ju skozi dve pomnilni celici 
    SYNC_INPUTS : process (clk)
    begin
        if rising_edge(clk) then
            if rst = '1' then
                kclk_1  <= '1';
                kclk_s  <= '1';
                kdata_1 <= '1';
                kdata_s <= '1';
            else
                kclk_1  <= kclk;
                kclk_s  <= kclk_1;
                kdata_1 <= kdata;
                kdata_s <= kdata_1;
            end if;
        end if;
    end process;

    -- Pomikalni register, ki ob we=1 pomakne vsebino
    -- v desno in v levi bit vpie bit s tipkovnice.
    -- kdata_s ---> [PAR | B7 | ... | B1 | B0]
    -- SIPO = Serial-In-Parallel-Out
    SIPO_REG : process (clk)
    begin
        if rising_edge(clk) then
            if rst = '1' then
                SIPO <= (others => '0');
            else
                if we = '1' then
                    -- operator konkatenacije je &
                    SIPO <= kdata_s & SIPO(8 downto 1);
                end if;
            end if;
        end if;
    end process;
end Behavioral;