----------------------------------------------------------------------------------
-- Krmilnik za tipkovnico po standardu PS/2
----------------------------------------------------------------------------------
-- Modul za generiranje pulza dolzine 1 periode notranje ure ob negativni fronti 
-- ure KCLK, ki jo krmili tipkovnica. Predpostavljamo, da je signal ure KCLK 
-- e sinhroniziran z notranjo uro (vhod KCLK_S).
--
-- verzija: 2021-11-23 
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity pulseGen is
	port (
		clk    : in  std_logic;
		rst    : in  std_logic;
		kclk_s : in  std_logic;
		pulse  : out std_logic);
end entity;

architecture Behavioral of pulseGen is
	-- STANJA 
	-- Definiramo nov "teven" podatkovni tip (enumerated type).
	-- Orodje za sintezo izbere najbolji nain kodiranja stanj
	type state_type is (S_IDLE, S_PULSE_ON, S_WAIT);
	-- Trenutno in naslednje stanje
	-- state = s(t) 
	-- next_state = s(t+1) 
	signal state, next_state : state_type;
	-- Izhod avtomata, ki se pomni
	signal output            : std_logic;
begin

	-- REGISTER STANJA
	-- Pomnilno (sekvenno vezje), ki vsako urino fronto osveuje 
	-- stanje avtomata, s(t) <= s(t+1), in skrbi tudi za pomnjenje izhoda,
	-- da se preprei pojav napetostnih konic (trava, ang. glitches).
	SYNC_PROC : process (clk)
	begin
		if rising_edge(clk) then
			if rst = '1' then
				-- Ponastavi stanje 
				state <= S_IDLE;
				pulse <= '0';
			else
				-- Pomnjenje stanja in izhoda
				state <= next_state;
				pulse <= output;
			end if;
		end if;
	end process;

	-- LOGIKA PREHAJANJA STANJ
	-- Naslednje stanje je odvisno od trenutnega stanja in vhoda
	NEXT_STATE_DECODE : process (kclk_s, state)
	begin
		-- Privzeto ostanemo v istem stanju.
		-- S tem stavkom se elegantno resimo "else" dela stavkov if spodaj.
		next_state <= state;
		-- Sedaj pokrijemo samo tiste primere, ko stanje spremenimo.
		case state is
			when S_IDLE =>
				if kclk_s = '0' then
					next_state <= S_PULSE_ON;
				end if;
			when S_PULSE_ON =>
				-- Ta prehod se zgodi ne glede na vsebino vhodov,
				-- zgodi pa se cez eno periodo ure.
				--if kclk_s='0' or kclk_s='1' then 
				next_state <= S_WAIT;
				--end if;
			when S_WAIT =>
				if kclk_s = '1' then
					next_state <= S_IDLE;
				end if;
			when others =>
				-- Sem nikoli ne pridemo;
				-- others je tu samo zato, da sintetizator ne bo mislil, 
				-- da hoemo pomnjenje. Vsak case potrebuje others.
				next_state <= S_IDLE;
		end case;
	end process;

	-- LOGIKA IZHODA 
	-- Izhod je odvisen od trenutnega stanja in vhoda (tip Mealy)
	OUTPUT_DECODE : process (kclk_s, state)
	begin
		-- Privzeta vrednost
		output <= '0';
		case state is
			when S_IDLE =>
				if kclk_s = '0' then
					output <= '1';
				end if;
			when S_PULSE_ON | S_WAIT =>
				output <= '0';
			when others =>
				-- Sem nikoli ne pridemo
				output <= '0';
		end case;
	end process;
end Behavioral;